@@ -31,7 +31,7 @@ typedef enum LogTarget{
         LOG_TARGET_SYSLOG,
         LOG_TARGET_SYSLOG_OR_KMSG,
 #if 0 /// Re-writing comment to indicate elogind LOG_TARGET_AUTO semantics change
-        LOG_TARGET_AUTO, /* console if stderr is tty, JOURNAL_OR_KMSG otherwise */
+        LOG_TARGET_AUTO, /* console if stderr is not journal, JOURNAL_OR_KMSG otherwise */
 #else // 0
         LOG_TARGET_AUTO, /* console if stderr is tty, SYSLOG_OR_KMSG otherwise */
 #endif // 0
@@ -63,9 +63,12 @@ void log_show_color(bool b);
 bool log_get_show_color(void) _pure_;
 void log_show_location(bool b);
 bool log_get_show_location(void) _pure_;
+void log_show_time(bool b);
+bool log_get_show_time(void) _pure_;
 
 int log_show_color_from_string(const char *e);
 int log_show_location_from_string(const char *e);
+int log_show_time_from_string(const char *e);
 
 LogTarget log_get_target(void) _pure_;
 int log_get_max_level_realm(LogRealm realm) _pure_;
@@ -78,7 +81,7 @@ int log_get_max_level_realm(LogRealm realm) _pure_;
  */
 
 assert_cc(STRLEN(__FILE__) > STRLEN(RELATIVE_SOURCE_PATH) + 1);
-#define PROJECT_FILE (__FILE__ + STRLEN(RELATIVE_SOURCE_PATH) + 1)
+#define PROJECT_FILE (&__FILE__[STRLEN(RELATIVE_SOURCE_PATH) + 1])
 
 int log_open(void);
 void log_close(void);
@@ -87,8 +90,11 @@ void log_forget_fds(void);
 #endif // 0
 
 void log_parse_environment_realm(LogRealm realm);
+void log_parse_environment_cli_realm(LogRealm realm);
 #define log_parse_environment() \
         log_parse_environment_realm(LOG_REALM)
+#define log_parse_environment_cli() \
+        log_parse_environment_cli_realm(LOG_REALM)
 
 int log_dispatch_internal(
                 int level,
@@ -236,12 +242,12 @@ void log_assert_failed_return_realm(
 #define log_full_errno(level, error, ...)                               \
         log_full_errno_realm(LOG_REALM, (level), (error), __VA_ARGS__)
 
-#define log_full(level, ...) log_full_errno((level), 0, __VA_ARGS__)
+#define log_full(level, ...) (void) log_full_errno((level), 0, __VA_ARGS__)
 
 int log_emergency_level(void);
 
 /* Normal logging */
-#define log_debug(...)     log_full(LOG_DEBUG,   __VA_ARGS__)
+#define log_debug(...)     log_full_errno(LOG_DEBUG, 0, __VA_ARGS__)
 #define log_info(...)      log_full(LOG_INFO,    __VA_ARGS__)
 #define log_notice(...)    log_full(LOG_NOTICE,  __VA_ARGS__)
 #define log_warning(...)   log_full(LOG_WARNING, __VA_ARGS__)
@@ -263,11 +269,12 @@ int log_emergency_level(void);
 #endif
 
 #if ENABLE_DEBUG_ELOGIND
-#  define log_debug_elogind_full(...) {   \
-        log_set_max_level(LOG_DEBUG);     \
-        log_full(LOG_DEBUG, __VA_ARGS__); \
-}
-#  define log_debug_elogind(fmt, ...) log_debug_elogind_full("(DEBUG) " fmt, __VA_ARGS__)
+#  define log_debug_elogind_full(...) do { \
+        log_set_max_level_realm(LOG_REALM, LOG_DEBUG);              \
+        log_full_errno_realm(LOG_REALM, LOG_DEBUG, 0, __VA_ARGS__); \
+} while(0)
+#  define log_debug_elogind(fmt, ...) \
+          log_debug_elogind_full("(DEBUG) %s:%d:%s: " fmt, PROJECT_FILE, __LINE__, __func__, __VA_ARGS__)
 #else
 #  define log_debug_elogind(...) do {} while (0)
 #endif // ENABLE_DEBUG_ELOGIND
@@ -314,11 +321,13 @@ void log_set_always_reopen_console(bool b);
  * desired as we want to reuse our logging streams. It is useful however  */
 void log_set_open_when_needed(bool b);
 
+#if 0 /// UNNEEDED by elogind
 /* If turned on, then we'll never use IPC-based logging, i.e. never log to syslog or the journal. We'll only log to
  * stderr, the console or kmsg */
 void log_set_prohibit_ipc(bool b);
 
 int log_dup_console(void);
+#endif // 0
 
 int log_syntax_internal(
                 const char *unit,
@@ -360,3 +369,4 @@ int log_syntax_invalid_utf8_internal(
 #define DEBUG_LOGGING _unlikely_(log_get_max_level() >= LOG_DEBUG)
 
 void log_setup_service(void);
+void log_setup_cli(void);
