@@ -6,7 +6,7 @@
 #if 0 /// UNNEEDED by elogind
 int mount_setup_early(void);
 #endif // 0
-int mount_setup(bool loaded_policy);
+int mount_setup(bool loaded_policy, bool leave_propagation);
 
 #if 0 /// UNNEEDED by elogind
 int mount_cgroup_controllers(void);
