@@ -10,12 +10,12 @@
 #include "bus-util.h"
 #include "string-util.h"
 
-_public_ int sd_bus_emit_signal(
+_public_ int sd_bus_emit_signalv(
                 sd_bus *bus,
                 const char *path,
                 const char *interface,
                 const char *member,
-                const char *types, ...) {
+                const char *types, va_list ap) {
 
         _cleanup_(sd_bus_message_unrefp) sd_bus_message *m = NULL;
         int r;
@@ -32,11 +32,7 @@ _public_ int sd_bus_emit_signal(
                 return r;
 
         if (!isempty(types)) {
-                va_list ap;
-
-                va_start(ap, types);
                 r = sd_bus_message_appendv(m, types, ap);
-                va_end(ap);
                 if (r < 0)
                         return r;
         }
@@ -44,7 +40,24 @@ _public_ int sd_bus_emit_signal(
         return sd_bus_send(bus, m, NULL);
 }
 
-_public_ int sd_bus_call_method_async(
+_public_ int sd_bus_emit_signal(
+                sd_bus *bus,
+                const char *path,
+                const char *interface,
+                const char *member,
+                const char *types, ...) {
+
+        va_list ap;
+        int r;
+
+        va_start(ap, types);
+        r = sd_bus_emit_signalv(bus, path, interface, member, types, ap);
+        va_end(ap);
+
+        return r;
+}
+
+_public_ int sd_bus_call_method_asyncv(
                 sd_bus *bus,
                 sd_bus_slot **slot,
                 const char *destination,
@@ -53,7 +66,7 @@ _public_ int sd_bus_call_method_async(
                 const char *member,
                 sd_bus_message_handler_t callback,
                 void *userdata,
-                const char *types, ...) {
+                const char *types, va_list ap) {
 
         _cleanup_(sd_bus_message_unrefp) sd_bus_message *m = NULL;
         int r;
@@ -70,11 +83,7 @@ _public_ int sd_bus_call_method_async(
                 return r;
 
         if (!isempty(types)) {
-                va_list ap;
-
-                va_start(ap, types);
                 r = sd_bus_message_appendv(m, types, ap);
-                va_end(ap);
                 if (r < 0)
                         return r;
         }
@@ -82,7 +91,28 @@ _public_ int sd_bus_call_method_async(
         return sd_bus_call_async(bus, slot, m, callback, userdata, 0);
 }
 
-_public_ int sd_bus_call_method(
+_public_ int sd_bus_call_method_async(
+                sd_bus *bus,
+                sd_bus_slot **slot,
+                const char *destination,
+                const char *path,
+                const char *interface,
+                const char *member,
+                sd_bus_message_handler_t callback,
+                void *userdata,
+                const char *types, ...) {
+
+        va_list ap;
+        int r;
+
+        va_start(ap, types);
+        r = sd_bus_call_method_asyncv(bus, slot, destination, path, interface, member, callback, userdata, types, ap);
+        va_end(ap);
+
+        return r;
+}
+
+_public_ int sd_bus_call_methodv(
                 sd_bus *bus,
                 const char *destination,
                 const char *path,
@@ -90,12 +120,13 @@ _public_ int sd_bus_call_method(
                 const char *member,
                 sd_bus_error *error,
                 sd_bus_message **reply,
-                const char *types, ...) {
+                const char *types, va_list ap) {
 
         _cleanup_(sd_bus_message_unrefp) sd_bus_message *m = NULL;
         int r;
 
         bus_assert_return(bus, -EINVAL, error);
+        bus_assert_return(bus = bus_resolve(bus), -ENOPKG, error);
         bus_assert_return(!bus_pid_changed(bus), -ECHILD, error);
 
         if (!BUS_IS_OPEN(bus->state)) {
@@ -108,11 +139,7 @@ _public_ int sd_bus_call_method(
                 goto fail;
 
         if (!isempty(types)) {
-                va_list ap;
-
-                va_start(ap, types);
                 r = sd_bus_message_appendv(m, types, ap);
-                va_end(ap);
                 if (r < 0)
                         goto fail;
         }
@@ -123,10 +150,30 @@ fail:
         return sd_bus_error_set_errno(error, r);
 }
 
-_public_ int sd_bus_reply_method_return(
-                sd_bus_message *call,
+_public_ int sd_bus_call_method(
+                sd_bus *bus,
+                const char *destination,
+                const char *path,
+                const char *interface,
+                const char *member,
+                sd_bus_error *error,
+                sd_bus_message **reply,
                 const char *types, ...) {
 
+        va_list ap;
+        int r;
+
+        va_start(ap, types);
+        r = sd_bus_call_methodv(bus, destination, path, interface, member, error, reply, types, ap);
+        va_end(ap);
+
+        return r;
+}
+
+_public_ int sd_bus_reply_method_returnv(
+                sd_bus_message *call,
+                const char *types, va_list ap) {
+
         _cleanup_(sd_bus_message_unrefp) sd_bus_message *m = NULL;
         int r;
 
@@ -147,11 +194,7 @@ _public_ int sd_bus_reply_method_return(
                 return r;
 
         if (!isempty(types)) {
-                va_list ap;
-
-                va_start(ap, types);
                 r = sd_bus_message_appendv(m, types, ap);
-                va_end(ap);
                 if (r < 0)
                         return r;
         }
@@ -159,6 +202,20 @@ _public_ int sd_bus_reply_method_return(
         return sd_bus_send(call->bus, m, NULL);
 }
 
+_public_ int sd_bus_reply_method_return(
+                sd_bus_message *call,
+                const char *types, ...) {
+
+        va_list ap;
+        int r;
+
+        va_start(ap, types);
+        r = sd_bus_reply_method_returnv(call, types, ap);
+        va_end(ap);
+
+        return r;
+}
+
 _public_ int sd_bus_reply_method_error(
                 sd_bus_message *call,
                 const sd_bus_error *e) {
@@ -186,14 +243,13 @@ _public_ int sd_bus_reply_method_error(
         return sd_bus_send(call->bus, m, NULL);
 }
 
-_public_ int sd_bus_reply_method_errorf(
+_public_ int sd_bus_reply_method_errorfv(
                 sd_bus_message *call,
                 const char *name,
                 const char *format,
-                ...) {
+                va_list ap) {
 
         _cleanup_(sd_bus_error_free) sd_bus_error error = SD_BUS_ERROR_NULL;
-        va_list ap;
 
         assert_return(call, -EINVAL);
         assert_return(call->sealed, -EPERM);
@@ -207,13 +263,27 @@ _public_ int sd_bus_reply_method_errorf(
         if (call->header->flags & BUS_MESSAGE_NO_REPLY_EXPECTED)
                 return 0;
 
-        va_start(ap, format);
         bus_error_setfv(&error, name, format, ap);
-        va_end(ap);
 
         return sd_bus_reply_method_error(call, &error);
 }
 
+_public_ int sd_bus_reply_method_errorf(
+                sd_bus_message *call,
+                const char *name,
+                const char *format,
+                ...) {
+
+        va_list ap;
+        int r;
+
+        va_start(ap, format);
+        r = sd_bus_reply_method_errorfv(call, name, format, ap);
+        va_end(ap);
+
+        return r;
+}
+
 _public_ int sd_bus_reply_method_errno(
                 sd_bus_message *call,
                 int error,
@@ -241,14 +311,13 @@ _public_ int sd_bus_reply_method_errno(
         return sd_bus_reply_method_error(call, &berror);
 }
 
-_public_ int sd_bus_reply_method_errnof(
+_public_ int sd_bus_reply_method_errnofv(
                 sd_bus_message *call,
                 int error,
                 const char *format,
-                ...) {
+                va_list ap) {
 
         _cleanup_(sd_bus_error_free) sd_bus_error berror = SD_BUS_ERROR_NULL;
-        va_list ap;
 
         assert_return(call, -EINVAL);
         assert_return(call->sealed, -EPERM);
@@ -262,13 +331,27 @@ _public_ int sd_bus_reply_method_errnof(
         if (call->header->flags & BUS_MESSAGE_NO_REPLY_EXPECTED)
                 return 0;
 
-        va_start(ap, format);
         sd_bus_error_set_errnofv(&berror, error, format, ap);
-        va_end(ap);
 
         return sd_bus_reply_method_error(call, &berror);
 }
 
+_public_ int sd_bus_reply_method_errnof(
+                sd_bus_message *call,
+                int error,
+                const char *format,
+                ...) {
+
+        va_list ap;
+        int r;
+
+        va_start(ap, format);
+        r = sd_bus_reply_method_errnofv(call, error, format, ap);
+        va_end(ap);
+
+        return r;
+}
+
 _public_ int sd_bus_get_property(
                 sd_bus *bus,
                 const char *destination,
@@ -283,6 +366,7 @@ _public_ int sd_bus_get_property(
         int r;
 
         bus_assert_return(bus, -EINVAL, error);
+        bus_assert_return(bus = bus_resolve(bus), -ENOPKG, error);
         bus_assert_return(isempty(interface) || interface_name_is_valid(interface), -EINVAL, error);
         bus_assert_return(member_name_is_valid(member), -EINVAL, error);
         bus_assert_return(reply, -EINVAL, error);
@@ -294,7 +378,10 @@ _public_ int sd_bus_get_property(
                 goto fail;
         }
 
-        r = sd_bus_call_method(bus, destination, path, "org.freedesktop.DBus.Properties", "Get", error, &rep, "ss", strempty(interface), member);
+        r = sd_bus_call_method(bus, destination, path,
+                               "org.freedesktop.DBus.Properties", "Get",
+                               error, &rep,
+                               "ss", strempty(interface), member);
         if (r < 0)
                 return r;
 
@@ -324,6 +411,7 @@ _public_ int sd_bus_get_property_trivial(
         int r;
 
         bus_assert_return(bus, -EINVAL, error);
+        bus_assert_return(bus = bus_resolve(bus), -ENOPKG, error);
         bus_assert_return(isempty(interface) || interface_name_is_valid(interface), -EINVAL, error);
         bus_assert_return(member_name_is_valid(member), -EINVAL, error);
         bus_assert_return(bus_type_is_trivial(type), -EINVAL, error);
@@ -368,6 +456,7 @@ _public_ int sd_bus_get_property_string(
         int r;
 
         bus_assert_return(bus, -EINVAL, error);
+        bus_assert_return(bus = bus_resolve(bus), -ENOPKG, error);
         bus_assert_return(isempty(interface) || interface_name_is_valid(interface), -EINVAL, error);
         bus_assert_return(member_name_is_valid(member), -EINVAL, error);
         bus_assert_return(ret, -EINVAL, error);
@@ -416,6 +505,7 @@ _public_ int sd_bus_get_property_strv(
         int r;
 
         bus_assert_return(bus, -EINVAL, error);
+        bus_assert_return(bus = bus_resolve(bus), -ENOPKG, error);
         bus_assert_return(isempty(interface) || interface_name_is_valid(interface), -EINVAL, error);
         bus_assert_return(member_name_is_valid(member), -EINVAL, error);
         bus_assert_return(ret, -EINVAL, error);
@@ -444,20 +534,20 @@ fail:
         return sd_bus_error_set_errno(error, r);
 }
 
-_public_ int sd_bus_set_property(
+_public_ int sd_bus_set_propertyv(
                 sd_bus *bus,
                 const char *destination,
                 const char *path,
                 const char *interface,
                 const char *member,
                 sd_bus_error *error,
-                const char *type, ...) {
+                const char *type, va_list ap) {
 
         _cleanup_(sd_bus_message_unrefp) sd_bus_message *m = NULL;
-        va_list ap;
         int r;
 
         bus_assert_return(bus, -EINVAL, error);
+        bus_assert_return(bus = bus_resolve(bus), -ENOPKG, error);
         bus_assert_return(isempty(interface) || interface_name_is_valid(interface), -EINVAL, error);
         bus_assert_return(member_name_is_valid(member), -EINVAL, error);
         bus_assert_return(signature_is_single(type, false), -EINVAL, error);
@@ -480,9 +570,7 @@ _public_ int sd_bus_set_property(
         if (r < 0)
                 goto fail;
 
-        va_start(ap, type);
         r = sd_bus_message_appendv(m, type, ap);
-        va_end(ap);
         if (r < 0)
                 goto fail;
 
@@ -496,6 +584,25 @@ fail:
         return sd_bus_error_set_errno(error, r);
 }
 
+_public_ int sd_bus_set_property(
+                sd_bus *bus,
+                const char *destination,
+                const char *path,
+                const char *interface,
+                const char *member,
+                sd_bus_error *error,
+                const char *type, ...) {
+
+        va_list ap;
+        int r;
+
+        va_start(ap, type);
+        r = sd_bus_set_propertyv(bus, destination, path, interface, member, error, type, ap);
+        va_end(ap);
+
+        return r;
+}
+
 _public_ int sd_bus_query_sender_creds(sd_bus_message *call, uint64_t mask, sd_bus_creds **creds) {
         sd_bus_creds *c;
 
