@@ -5,11 +5,9 @@
 #include <sys/types.h>
 
 #if 0 /// elogind needs the systems udev header
-#include "libudev.h"
-#else
+#else // 0
 #include <libudev.h>
 #endif // 0
-
 #if HAVE_ACL
 
 int devnode_acl(const char *path,
@@ -17,8 +15,7 @@ int devnode_acl(const char *path,
                 bool del, uid_t old_uid,
                 bool add, uid_t new_uid);
 
-int devnode_acl_all(struct udev *udev,
-                    const char *seat,
+int devnode_acl_all(const char *seat,
                     bool flush,
                     bool del, uid_t old_uid,
                     bool add, uid_t new_uid);
@@ -31,8 +28,7 @@ static inline int devnode_acl(const char *path,
         return 0;
 }
 
-static inline int devnode_acl_all(struct udev *udev,
-                                  const char *seat,
+static inline int devnode_acl_all(const char *seat,
                                   bool flush,
                                   bool del, uid_t old_uid,
                                   bool add, uid_t new_uid) {
