@@ -1,7 +1,6 @@
 /* SPDX-License-Identifier: LGPL-2.1+ */
 #pragma once
 
-
 typedef enum DeviceType DeviceType;
 typedef struct SessionDevice SessionDevice;
 
@@ -34,7 +33,7 @@ void session_device_complete_pause(SessionDevice *sd);
 
 void session_device_resume_all(Session *s);
 void session_device_pause_all(Session *s);
-unsigned int session_device_try_pause_all(Session *s);
+unsigned session_device_try_pause_all(Session *s);
 
 int session_device_save(SessionDevice *sd);
 void session_device_attach_fd(SessionDevice *sd, int fd, bool active);
