@@ -2,6 +2,7 @@
 #pragma once
 
 #include <stdbool.h>
+#include <sys/stat.h>
 
 #include "sd-bus.h"
 #include "sd-device.h"
@@ -12,6 +13,7 @@
 #include "list.h"
 #include "set.h"
 #include "time-util.h"
+#include "user-record.h"
 
 typedef struct Manager Manager;
 
@@ -23,6 +25,7 @@ typedef struct Manager Manager;
 /// Additional includes needed by elogind
 #include "cgroup-util.h"
 #include "elogind.h"
+#include "musl_missing.h"
 
 #if 1 /// elogind has to ident itself
 #define MANAGER_IS_SYSTEM(m)   (  (m)->is_system)
@@ -37,7 +40,7 @@ struct Manager {
         Hashmap *seats;
         Hashmap *sessions;
         Hashmap *sessions_by_leader;
-        Hashmap *users;
+        Hashmap *users;  /* indexed by UID */
         Hashmap *inhibitors;
         Hashmap *buttons;
         Hashmap *brightness_writers;
@@ -111,23 +114,38 @@ struct Manager {
         char *action_job;
 #else // 0
         /* Suspension and hibernation can be disabled in logind.conf. */
-        bool allow_suspend, allow_hibernation, allow_suspend_then_hibernate, allow_hybrid_sleep;
+        bool allow_suspend;         /* AllowSuspend */
+        bool allow_hibernate;       /* AllowHibernation */
+        bool allow_s2h;             /* AllowSuspendThenHibernate */
+        bool allow_hybrid_sleep;    /* AllowHybridSleep */
+
+        char **suspend_modes;       /* SuspendMode */
+        char **suspend_states;      /* SuspendState */
+        char **hibernate_modes;     /* HibernateMode */
+        char **hibernate_states;    /* HibernateState */
+        char **hybrid_modes;        /* HybridSleepMode */
+        char **hybrid_states;       /* HybridSleepState */
+
+        usec_t hibernate_delay_sec; /* HibernateDelaySec */
 
         /* If an admin puts scripts into SYSTEM_SLEEP_PATH and/or
            SYSTEM_POWEROFF_PATH that fail, the ongoing suspend/poweroff
            action will be cancelled if any of these are set to true. */
         bool allow_poweroff_interrupts, allow_suspend_interrupts;
+        bool broadcast_poweroff_interrupts, broadcast_suspend_interrupts;
         bool callback_failed, callback_must_succeed;
 
+        /* Allow elogind to put Nvidia cards to sleep */
+        bool handle_nvidia_sleep;
+
         /* If a shutdown/suspend was delayed due to a inhibitor this
            contains the action we are supposed to perform after the
            delay is over */
         HandleAction pending_action;
 
-        char **suspend_state,      **suspend_mode;
-        char **hibernate_state,    **hibernate_mode;
-        char **hybrid_sleep_state, **hybrid_sleep_mode;
-        usec_t hibernate_delay_sec;
+        /* To allow elogind to put nvidia cards to sleep on suspend/hibernate,
+           we store the users uid to get the right VT information */
+        uid_t scheduled_sleep_uid;
 #endif // 0
         sd_event_source *inhibit_timeout_source;
 
@@ -170,22 +188,28 @@ struct Manager {
         sd_event_source *lid_switch_ignore_event_source;
 
         uint64_t runtime_dir_size;
-        uint64_t user_tasks_max;
+        uint64_t runtime_dir_inodes;
         uint64_t sessions_max;
         uint64_t inhibitors_max;
+
+        char **efi_boot_loader_entries;
+        bool efi_boot_loader_entries_set;
+
+        char *efi_loader_entry_one_shot;
+        struct stat efi_loader_entry_one_shot_stat;
 };
 
 void manager_reset_config(Manager *m);
 int manager_parse_config_file(Manager *m);
 
-int manager_add_device(Manager *m, const char *sysfs, bool master, Device **_device);
-int manager_add_button(Manager *m, const char *name, Button **_button);
-int manager_add_seat(Manager *m, const char *id, Seat **_seat);
-int manager_add_session(Manager *m, const char *id, Session **_session);
-int manager_add_user(Manager *m, uid_t uid, gid_t gid, const char *name, const char *home, User **_user);
-int manager_add_user_by_name(Manager *m, const char *name, User **_user);
-int manager_add_user_by_uid(Manager *m, uid_t uid, User **_user);
-int manager_add_inhibitor(Manager *m, const char* id, Inhibitor **_inhibitor);
+int manager_add_device(Manager *m, const char *sysfs, bool master, Device **ret_device);
+int manager_add_button(Manager *m, const char *name, Button **ret_button);
+int manager_add_seat(Manager *m, const char *id, Seat **ret_seat);
+int manager_add_session(Manager *m, const char *id, Session **ret_session);
+int manager_add_user(Manager *m, UserRecord *ur, User **ret_user);
+int manager_add_user_by_name(Manager *m, const char *name, User **ret_user);
+int manager_add_user_by_uid(Manager *m, uid_t uid, User **ret_user);
+int manager_add_inhibitor(Manager *m, const char* id, Inhibitor **ret_inhibitor);
 
 int manager_process_seat_device(Manager *m, sd_device *d);
 int manager_process_button_device(Manager *m, sd_device *d);
@@ -208,8 +232,6 @@ int manager_read_utmp(Manager *m);
 void manager_connect_utmp(Manager *m);
 void manager_reconnect_utmp(Manager *m);
 
-extern const sd_bus_vtable manager_vtable[];
-
 /* gperf lookup function */
 const struct ConfigPerfItem* logind_gperf_lookup(const char *key, GPERF_LEN_TYPE length);
 
@@ -220,3 +242,5 @@ CONFIG_PARSER_PROTOTYPE(config_parse_tmpfs_size);
 
 int manager_setup_wall_message_timer(Manager *m);
 bool logind_wall_tty_filter(const char *tty, void *userdata);
+
+int manager_read_efi_boot_loader_entries(Manager *m);
