@@ -87,7 +87,7 @@ static void test_unhexmem_one(const char *s, size_t l, int retval) {
                         l = strlen(s);
 
                 assert_se(hex = hexmem(mem, len));
-                answer = strndupa(s, l);
+                answer = strndupa(strempty(s), l);
                 assert_se(streq(delete_chars(answer, WHITESPACE), hex));
         }
 }
@@ -236,7 +236,6 @@ static void test_unbase32hexmem(void) {
         test_unbase32hexmem_one("CPNMUOG=", false, -EINVAL, NULL);
         test_unbase32hexmem_one("CPNMUOJ1E8======", false, -EINVAL, NULL);
 
-        test_unbase32hexmem_one("A", false, -EINVAL, NULL);
         test_unbase32hexmem_one("A", false, -EINVAL, NULL);
         test_unbase32hexmem_one("AAA", false, -EINVAL, NULL);
         test_unbase32hexmem_one("AAAAAA", false, -EINVAL, NULL);
