@@ -4,7 +4,7 @@
 
 #include "process-util.h"
 #include "set.h"
-//#include "tests.h"
+#include "tests.h"
 
 #define NUM 100
 
@@ -42,9 +42,7 @@ static void test_one(const char *val) {
 }
 
 int main(int argc, char *argv[]) {
-        log_set_max_level(LOG_DEBUG);
-        log_parse_environment();
-        log_open();
+        test_setup_logging(LOG_DEBUG);
 
         test_one("0");
         /* The value $SYSTEMD_MEMPOOL= is cached. So the following
